/*
 * dvrctl.c
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 */

static const char *svnid = "$Id: dvrctl.c 56 2008-08-28 12:37:00Z epipe $";

#include "snavtclient.h"

/* the following are set by dvr_login() */
char * dvr_cookie = NULL;
char * dvr_firmware = NULL;
char * dvr_productid = NULL;
char * dvr_macaddress = NULL;
char * dvr_hostname = NULL;

enum setany { setreso, setqual, setdate, setrs485 };

int
dvr_setany(enum setany mode, int arg, char *arg2)
{
  TCPsocket sd;
  char *template;
  char *cp;
  char *http_hdr, *http_data;

  switch (mode) {
    case setreso:
      template = "/Setresolution.cgi?Cookie=%s&RES=%d";
      break;
    case setqual:
      template = "/Setquality.cgi?Cookie=%s&QUA=%d";
      break;
    case setdate:
      template = "/SetDvrDate.cgi?Cookie=%s&%s";
      break;
    case setrs485:
      template = "/RS485Cmd.cgi?Control=787E&MASTER=1&Cookie=%s";
      break;
    default:
      return -1;
  } /* switch (setany) */

  if (!dvr_cookie)
    return -1;

  if (!(sd = SDLNet_TCP_Open(&dvr_ip))) {
    fprintf(stderr, "%s: SDLNet_TCP_Open: %s\n", argv0, SDLNet_GetError());
    return -1;
  }
  if (mode == setrs485) {
    uint32_t tmp = arg;

    h2le32(&tmp);

    if (asprintf(&cp, template, dvr_cookie) < 0) {
      fprintf(stderr, "%s: asprintf: %s\n", argv0, strerror(errno));
      SDLNet_TCP_Close(sd);
      return -1;
    }
    if (send_http_request(sd, cp, &tmp, 4) < 0) {
      free(cp);
      SDLNet_TCP_Close(sd);
      return -1;
    }
  } else {
    if (asprintf(&cp, template, dvr_cookie, 
        ((mode == setdate) ? (arg2) : (arg))) < 0) {
      fprintf(stderr, "%s: asprintf: %s\n", argv0, strerror(errno));
      SDLNet_TCP_Close(sd);
      return -1;
    }
    if (send_http_request(sd, cp, NULL, 0) < 0) {
      free(cp);
      SDLNet_TCP_Close(sd);
      return -1;
    }
  }
  free(cp);
  http_hdr = NULL;
  if (recv_http_response(sd, &http_hdr, &http_data) < 0) {
    if (http_hdr) {
      free(http_hdr);
    }
    SDLNet_TCP_Close(sd);
    return -1;
  }
  /* XXX not sure if there is anything interesting in the results to check */
     
  free(http_hdr);
  SDLNet_TCP_Close(sd);

  return 0;
}

int 
dvr_setreso(int arg)
{
  return dvr_setany(setreso, arg, NULL);
}

int 
dvr_setqual(int arg)
{
  return dvr_setany(setqual, arg, NULL);
}

int 
dvr_setchan(int arg)
{
  return dvr_ctl(DVRCTL_CH(arg));
}

int
dvr_setdatenow()
{
  char buf[256];
  time_t now;

  time(&now);

  strftime(buf, sizeof buf, 
       "YEAR=%Y&MONTH=%m&DAY=%d&DAYOFWEEK=%u&HOUR=%H&MINUTE=%M&SECOND=%S"
       "&CHDATE=1",
       localtime(&now));

  return dvr_setany(setdate, 0, buf);
}

int 
dvr_setrs485(uint32_t arg)
{
  return dvr_setany(setrs485, arg, NULL);
}

int
dvr_ctl(int op)
{
  uint32_t dvrop;

  if (DVRCTL_IS_CH(op)) {
#ifdef DEBUGXXX
    printf("Channel %d selected\n", DVRCTL_GET_CH(op));
#endif
    dvrop = 0x32010001 | ((0x36 + DVRCTL_GET_CH(op)) << 8);
  } else {
#ifdef DEBUGXXX
    printf("DVR Control key %x pressed\n", op);
#endif
    switch (op) {
      case DVRCTL_MENU:		dvrop = 0x32011801; break;
      case DVRCTL_UP:		dvrop = 0x32010001; break;
      case DVRCTL_DOWN:		dvrop = 0x32010101; break;
      case DVRCTL_LEFT:		dvrop = 0x32010201; break;
      case DVRCTL_RIGHT:	dvrop = 0x32010301; break;
      case DVRCTL_ENTER:	dvrop = 0x32010401; break;
      case DVRCTL_SEARCH:	dvrop = 0x32011f01; break;
      case DVRCTL_ZOOM:		dvrop = 0x32012701; break;
      case DVRCTL_RECORD:	dvrop = 0x32011b01; break;
      case DVRCTL_PLAY:		dvrop = 0x32011301; break;
      case DVRCTL_STOP:		dvrop = 0x32011401; break;
      case DVRCTL_SLOW:		dvrop = 0x32011d01; break;
      case DVRCTL_PAUSE:	dvrop = 0x32011c01; break;
      case DVRCTL_FAST:		dvrop = 0x32010301; break;
      case DVRCTL_REW:		dvrop = 0x32010201; break;
      case DVRCTL_4CUT:		dvrop = 0x32011a01; break;
      case DVRCTL_9CUT:		dvrop = 0x32011901; break;
      case DVRCTL_16CUT:	dvrop = 0x32011001; break;
      case DVRCTL_MINUS:	dvrop = 0x32011701; break;
      case DVRCTL_PLUS:		dvrop = 0x32011601; break;
      case DVRCTL_SET:		dvrop = 0x32011501; break;
      case DVRCTL_SEQUENCE:	dvrop = 0x32012501; break;
      case DVRCTL_AUMINUS:	dvrop = 0x32012a01; break;
      case DVRCTL_AUPLUS:	dvrop = 0x32012901; break;
      case DVRCTL_TIMER:	dvrop = 0x32011e01; break;
      case DVRCTL_USBCON:	dvrop = 0x32012101; break;
      case DVRCTL_USBDIS:	dvrop = 0x32012201; break;
      case DVRCTL_PTZ:		dvrop = 0x32012c01; break;
      case DVRCTL_PTZGOTO:	dvrop = 0x32013201; break;
      case DVRCTL_0:		dvrop = 0x32010501; break;
      case DVRCTL_1:		dvrop = 0x32010601; break;
      case DVRCTL_2:		dvrop = 0x32010701; break;
      case DVRCTL_3:		dvrop = 0x32010801; break;
      case DVRCTL_4:		dvrop = 0x32010901; break;
      case DVRCTL_5:		dvrop = 0x32010a01; break;
      case DVRCTL_6:		dvrop = 0x32010b01; break;
      case DVRCTL_7:		dvrop = 0x32010c01; break;
      case DVRCTL_8:		dvrop = 0x32010d01; break;
      case DVRCTL_9:		dvrop = 0x32010e01; break;

      default:
#ifdef DEBUG
        printf("UNKNOWN DVR Control key %x received\n", op);
#endif
        return -1;
    }
  }
#ifdef DEBUGXXX
  printf("Sending RS485 code %x\n", dvrop);
#endif

  return dvr_setrs485(dvrop);
}

char *
dvr_login() 
{
  TCPsocket sd;
  char *http_hdr, *http_data;
  char *tok, *cp;

  if (!(sd = SDLNet_TCP_Open(&dvr_ip))) {
    fprintf(stderr, "%s: SDLNet_TCP_Open: %s\n", argv0, SDLNet_GetError());
    return NULL;
  }
  dvr_cookie = NULL;
  dvr_firmware = NULL;
  dvr_productid = NULL;
  dvr_macaddress = NULL;
  dvr_hostname = NULL;

  if (asprintf(&cp, "/Login.cgi?Username=%s&Password=%s", dvr_user, dvr_pass)
      < 0) {
    fprintf(stderr, "%s: asprintf: %s\n", argv0, strerror(errno));
    SDLNet_TCP_Close(sd);
    return NULL;
  }
  if (send_http_request(sd, cp, NULL, 0) < 0) {
    free(cp);
    SDLNet_TCP_Close(sd);
    return NULL;
  }
  free(cp);
  http_hdr = NULL;
  if (recv_http_response(sd, &http_hdr, &http_data) < 0) {
    if (http_hdr) {
      free(http_hdr);
    }
    SDLNet_TCP_Close(sd);
    return NULL;
  }

  cp = http_hdr;
  while ((tok = strsep(&cp, "\n")) != NULL) {
    if (strlen(tok) > 0 && tok[strlen(tok) - 1] == '\r') {
      tok[strlen(tok) - 1] = '\0';
    }
    if (snmode == mode_info) {
      puts(tok);
    }
  }

  /* Returned http_data contains a single non-terminated string similar
   * to the following: 
   *     Session-ID=77C30A73D5A7CD0361A1F8860FAAFD;Firmware-Version=2020;
   *     Product-ID=785E;MAC-Address=000e53050808;Host-Name=AV732E;
   * (all on single line) 
   */

  cp = http_data;
  while ((tok = strsep(&cp, ";")) != NULL) {
    if (snmode == mode_info) {
      puts(tok);
    }
    if (!strncmp(tok, "Session-ID=", strlen("Session-ID="))) {
      dvr_cookie = strdup(tok + strlen("Session-ID="));
    }
    if (!strncmp(tok, "Firmware-Version=", strlen("Firmware-Version="))) {
      dvr_firmware = strdup(tok + strlen("Firmware-Version="));
    }
    if (!strncmp(tok, "Product-ID=", strlen("Product-ID="))) {
      dvr_productid = strdup(tok + strlen("Product-ID="));
    }
    if (!strncmp(tok, "MAC-Address=", strlen("MAC-Address="))) {
      dvr_macaddress = strdup(tok + strlen("MAC-Address="));
    }
    if (!strncmp(tok, "Host-Name=", strlen("Host-Name="))) {
      dvr_hostname = strdup(tok + strlen("Host-Name="));
    }
  }

  free(http_hdr);
  SDLNet_TCP_Close(sd);

  return dvr_cookie;
}

/* eof */
