/*
 * dvrvideo.c
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 */

static const char *svnid = "$Id: dvrvideo.c 36 2008-07-11 04:32:09Z epipe $";

#include "snavtclient.h"

dvrvideohdrdesc_t dvrvideohdrdesc[] = {
#define HDR_SIZE 0
  { "size",			0x00000001,  4, 1 },
#define HDR_HANDLER_USER 1
  { "handler_user",		0x00000002, 16, 0 },
#define HDR_ALARM 2
  { "alarm",			0x00000004,  2, 1 },
#define HDR_RES 3
  { "res",			0x00000008,  1, 1 },
#define HDR_QUA 4
  { "qua",			0x00000010,  1, 1 },
#define HDR_TIMESTAMP 5
  { "timestamp",		0x00000020,  4, 1 },
#define HDR_ONLINE_USER 6
  { "online_user",		0x00000040,  1, 1 },
#define HDR_ALARM_TRIGGER 7
  { "alarm_trigger",		0x00000080,  2, 1 },
#define HDR_MOTION_DETECT 8
  { "motion_detect",		0x00000100,  2, 1 },
#define HDR_VIDEO_LOSS 9
  { "video_loss",		0x00000200,  2, 1 },
#define HDR_RETR_DOWNLOAD_FILE 10
  { "retr_download_file",	0x00000400,  0, 0 },
#define HDR_CH 11
  { "ch",			0x00000800,  1, 1 },
#define HDR_NO_SIGNAL 12
  { "no_signal",		0x00001000,  0, 0 },
#define HDR_SERVER_TIME 13
  { "server_time",		0x00002000,  4, 1 },
  { NULL,			         0,  0, 0 }
};

char * dumpjpeg = NULL;
char * dumpjpegsuffix = NULL;
int    dumpcount = 1;
int    dumpinterval = 1;
int    dumpseq = 0;
time_t dumplast = 0;

size_t  dvrvideohdrmaxlen = 0;
int     dvrvideohdrcount = 0;
uint32_t dvrvideohdrallbits = 0;

int quit = 0;

/* subroutines */

char *
dumpjpegframe(void * data, size_t datalen)
{
  static char *fn = NULL; /* XXX ugly */
  static int fn_len = 0;
  int fd;
  char *tmpfn;

  if (dumpseq) {
    if (!fn) {
      fn_len = strlen(dumpjpeg) + 6 + strlen(dumpjpegsuffix) + 1;
      if (!(fn = malloc(fn_len))) {
        return NULL;
      }
    }
    snprintf(fn, fn_len, "%s%06d%s", dumpjpeg, dumpseq, dumpjpegsuffix);
  } else {
    fn = dumpjpeg;
  }
  if (asprintf(&tmpfn, "%s.tmp", dumpjpeg) < 0) {
    return NULL;
  }
  unlink(tmpfn);
  if ((fd = open(tmpfn, O_WRONLY | O_CREAT | O_TRUNC | O_EXCL, 0666)) < 0) {
    free(tmpfn);
    return NULL;
  }
  if (write(fd, data, datalen) < datalen) {
    close(fd);
    unlink(tmpfn); 
    free(tmpfn);
    return NULL;
  }
  if ((close(fd) < 0) 
      || (rename(tmpfn, fn) < 0)) {
    unlink(tmpfn);
    free(tmpfn);
    return NULL;
  }
  free(tmpfn);
 
  if (dumpseq) {
    dumpseq++;
  }
  return fn;
}

int
dvr_getvideo_init(void) 
{
  dvrvideohdrdesc_t * hp;

  dvrvideohdrcount = 0;
  dvrvideohdrallbits = 0;
  dvrvideohdrmaxlen = 0;

  for (hp = dvrvideohdrdesc; hp->name != NULL; hp++) {
    dvrvideohdrcount++;
    dvrvideohdrallbits |= hp->bit;
    dvrvideohdrmaxlen  += hp->len;
  }
  return 1;
}

int
dvr_getvideo(void) 
{
  TCPsocket sd;
  char *req;
  char *hdrbuf;
  char *framebuf = NULL;
  size_t framebufsize = 0;
  void **hdrptrs;
  int frames = 0;

  if (!(hdrbuf = malloc(dvrvideohdrmaxlen))) {
    fprintf(stderr, "%s: malloc: %s\n", argv0, strerror(errno));
    return 0;
  }
  if (!(hdrptrs = malloc(dvrvideohdrcount * sizeof (void *)))) {
    fprintf(stderr, "%s: malloc: %s\n", argv0, strerror(errno));
    return 0;
  }
  if (asprintf(&req, "/Getvideo.cgi?Cookie=%s", dvr_cookie) < 0) {
    fprintf(stderr, "%s: malloc: %s\n", argv0, strerror(errno));
    free(hdrptrs);
    free(hdrbuf);
    return 0;
  }
  if (!(sd = SDLNet_TCP_Open(&dvr_ip))) {
    fprintf(stderr, "%s: SDLNet_TCP_Open: %s\n", argv0, SDLNet_GetError());
    free(req);
    free(hdrptrs);
    free(hdrbuf);
    return 0;
  }
  if (send_http_request(sd, req, NULL, 0) < 0) {
    SDLNet_TCP_Close(sd);
    free(req);
    free(hdrptrs);
    free(hdrbuf);
    return 0;
  }
  free(req);

  if (!(framebuf = malloc(RECVBUFFER))) {
    fprintf(stderr, "%s: malloc: %s\n", argv0, strerror(errno));
    SDLNet_TCP_Close(sd);
    free(hdrptrs);
    free(hdrbuf);
    return 0;
  }
  framebufsize = RECVBUFFER;

  quit = 0;

  while (!quit) {
    uint32_t flags;
    dvrvideohdrdesc_t * hp;
    char * cp;
    size_t hdrlen = 0;
    size_t framelen = 0;
    int i;

    if (mySDLNet_TCP_Recv(sd, &flags, sizeof flags) != sizeof flags) {
      fprintf(stderr, "%s: SDLNet_TCP_Recv(2): %s\n", argv0, SDLNet_GetError());
      SDLNet_TCP_Close(sd);
      free(framebuf);
      free(hdrptrs);
      free(hdrbuf);
      return 0;
    }
    le2h32(&flags);

    if (flags & ~dvrvideohdrallbits) {
      fprintf(stderr, "%s: received unknown flags from DVR: %08x\n", 
          argv0, flags);
      SDLNet_TCP_Close(sd);
      free(framebuf);
      free(hdrptrs);
      free(hdrbuf);
      return 0;
    }

    hdrlen = 0;
    for (hp = dvrvideohdrdesc; hp->name != NULL; hp++) {
      if (flags & hp->bit) {
        hdrlen += hp->len;
      }
    }
#ifdef DEBUGXXX
    printf("header len: %d\n", hdrlen);
#endif
    if (mySDLNet_TCP_Recv(sd, hdrbuf, hdrlen) != hdrlen) {
      fprintf(stderr, "%s: SDLNet_TCP_Recv(3): %s\n", argv0, SDLNet_GetError());
      free(framebuf);
      free(hdrptrs);
      free(hdrbuf);
      SDLNet_TCP_Close(sd);
      return 0;
    }
    cp = hdrbuf;
    i = 0;
    for (hp = dvrvideohdrdesc; hp->name != NULL; hp++) {
      if (flags & hp->bit) {
        if (hp->swap) {
          switch (hp->len) {
            case 4:
              le2h32(cp);
              break;
            case 2:
              le2h16(cp);
              break;
            case 1:
            case 0:
              break;
            default:
              assert(hp->len != 0);
          }
        }
        hdrptrs[i] = cp;
        cp += hp->len;
      } else {
        hdrptrs[i] = NULL;
      }
      i++;
    }
    if (hdrptrs[HDR_SIZE]) {
      framelen = *((size_t *) hdrptrs[HDR_SIZE]);
    } else {
      framelen = 0;
    }
    if (framelen) {
      if (framelen > framebufsize) {
        while (framelen > framebufsize) {
          framebufsize += RECVBUFFER;
        }
#ifdef DEBUGXXX
        printf("realloc(framebuf, %d)\n", framebufsize);
#endif
        if (!(framebuf = realloc(framebuf, framebufsize))) {
          SDLNet_TCP_Close(sd);
          free(framebuf);
          free(hdrptrs);
          free(hdrbuf);
        }
      }
#ifdef DEBUGXXX
      printf("frame len: %d\n", framelen);
#endif

      if (mySDLNet_TCP_Recv(sd, framebuf, framelen) != framelen) {
        fprintf(stderr, "%s: SDLNet_TCP_Recv(4): %s\n", argv0, SDLNet_GetError());
        free(framebuf);
        free(hdrptrs);
        free(hdrbuf);
        SDLNet_TCP_Close(sd);
        return 0;
      }
      switch (snmode) {
        case mode_dumpjpeg:
          if (!dumpinterval || time(NULL) - dumplast >= dumpinterval) {
            dumpjpegframe(framebuf, framelen);
            dumplast = time(NULL);
          
            if (dumpcount == 1 || dumpseq > dumpcount) {
              quit = 1;
            }
          }
          break;
        case mode_show:
          if (opt_showskip && frames % (opt_showskip + 1))
            break;
          showjpegframe(framebuf, framelen);
          break;
        default:
          assert(("unknown mode", 0));
      } /* switch (snmode) */
      frames++;
    } /* if (framelen > 0) */
  } /* while (1) */

  free(hdrptrs);
  free(hdrbuf);
  SDLNet_TCP_Close(sd);

  return frames;
}

/* eof */
