/*
 * http.c
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 */

static const char *svnid = "$Id: http.c 51 2008-07-18 01:35:56Z epipe $";

#include "snavtclient.h"

/* subroutines */

/* SDLNet_TCP_Recv() returns short reads, so we implement
   this wrapper to ensure that we always get as many bytes 
   as we asked for, even if we block */

/* XXX BUG: now we get stuck some times, must kill process... */

int
mySDLNet_TCP_Recv(TCPsocket sock, void *data, int maxlen)
{
  int len = 0;
  int ret;
  char * buf = data;

  while ((ret = SDLNet_TCP_Recv(sock, buf, maxlen - len)) >= 0) {
    len += ret;
    buf += ret;

    if (len >= maxlen || ret == 0) {
      return len;
    }
  }
  /* read error */
  return -1;
}

size_t 
send_http_request(TCPsocket sd, char * req, void * data, size_t datalen)
{
  char *buf;
  size_t hlen, tlen;
 
  asprintf(&buf, "GET %s HTTP/1.0\r\n"
                 "Accept: */*\r\n"
                 "Content-Length: %zd\r\n"
                 "User-Agent: %s/%s\r\n"
                 "\r\n", 
      req, datalen, PACKAGE, PACKAGE_VERSION);

  hlen = strlen(buf);
  tlen = hlen;

  if (datalen > 0) {
    /* 
       Looks like the HTTP data portion MUST arrive to DVR inside the same
       TCP packet as the HTTP header. If the data part arrives in a separate 
       packet, AV TECH DVR will just ignore it... 

       Great TCP implementation, AV TECH guys! Very professional! NOT!

       To guarantee this behaviour on client side we would have to send 
       raw IP packets. Luckily in real life most OSes send a single 
       TCP packet as a result of single write() (if the data size is small 
       enough). But by specification OS may split the write() up to 
       separate TCP packets. TCP is a stram based protocol. I suppose 
       AV TECH got confused if they were implementing TCP or UDP?
    */

    if (!(buf = realloc(buf, hlen + datalen))) {
      fprintf(stderr, "%s: %s\n", argv0, strerror(errno));
      return -1;
    }
    memcpy(((uint8_t *) buf) + hlen, data, datalen);
    tlen += datalen;
  }

  if (SDLNet_TCP_Send(sd, buf, tlen) < tlen) {
    free(buf);
    fprintf(stderr, "%s: SDLNet_TCP_Send: %s\n", argv0, SDLNet_GetError());
    return -1;
  }
  free(buf);
  return tlen;
}

size_t 
recv_http_response(TCPsocket sd, char **hdr, char **data)
{
  char *buf, *rp;
  size_t len = 0;
  int ret;

  if (!(buf = malloc(len + RECVBUFFER + 1))) {
    fprintf(stderr, "%s: %s\n", argv0, strerror(errno));
    return -1;
  }
  rp = buf;
  while ((ret = mySDLNet_TCP_Recv(sd, rp, RECVBUFFER)) == RECVBUFFER) {
    len += ret;
    if (!(buf = realloc(buf, len + RECVBUFFER + 1))) {
      fprintf(stderr, "%s: %s\n", argv0, strerror(errno));
      return -1;
    }
    rp = buf + len;
  }

  if (ret < 0) {
    fprintf(stderr, "%s: SDLNet_TCP_Recv(1): %s\n", argv0, SDLNet_GetError());
    return -1;
  }
  len += ret;
  buf[len] = '\0';
  
  *hdr = buf;
  *data = buf;

  if ((*data = strstr(*hdr, "\r\n\r\n")) == NULL) {
    fprintf(stderr, "%s: invalid HTTP response\n", argv0);
    return -1;
  }
  *data[0] = '\0';	/* nul terminate header part */ 
  *data += 4;		/* skip separator */

  /* checking is pretty simple but should be enough for our needs... */

  if (strncmp(*hdr, "HTTP/1.0 200 ", strlen("HTTP/1.0 200 "))
      && strncmp(*hdr, "HTTP/1.1 200 ", strlen("HTTP/1.1 200 "))) {
    fprintf(stderr, "%s: HTTP request not successful\n", argv0);
    return -1;
  }
  
  return len;
}

/* eof */
