/*
 * sdlshow.c
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 */

static const char *svnid = "$Id: sdlshow.c 54 2008-08-16 05:59:01Z epipe $";

#include "snavtclient.h"

static SDL_Surface * screen = NULL;
static int prevw = 0;
static int prevh = 0;

static unsigned long sdlframes = 0;
static Uint32 sdlticks = 0;
static int notvisible = 0;

void
sdlkeypress(SDL_KeyboardEvent * ke)
{
  switch (ke->keysym.sym) {
    case SDLK_ESCAPE:
      quit++;
      break;

    case SDLK_1:	dvr_setchan(1); break;
    case SDLK_2:	dvr_setchan(2); break;
    case SDLK_3:	dvr_setchan(3); break;
    case SDLK_4:	dvr_setchan(4); break;
    case SDLK_5:	dvr_setchan(5); break;
    case SDLK_6:	dvr_setchan(6); break;
    case SDLK_7:	dvr_setchan(7); break;
    case SDLK_8:	dvr_setchan(8); break;
    case SDLK_9:	dvr_setchan(9); break;
    case SDLK_0:	dvr_setchan(10); break;
    case SDLK_F1:	dvr_setchan(11); break;
    case SDLK_F2:	dvr_setchan(12); break;
    case SDLK_F3:	dvr_setchan(13); break;
    case SDLK_F4:	dvr_setchan(14); break;
    case SDLK_F5:	dvr_setchan(15); break;
    case SDLK_F6:	dvr_setchan(16); break;

    case SDLK_F10:	dvr_ctl(DVRCTL_4CUT);	break;
    case SDLK_F11:	dvr_ctl(DVRCTL_9CUT);	break;
    case SDLK_F12:	dvr_ctl(DVRCTL_16CUT);	break;
    case SDLK_TAB:	dvr_ctl(DVRCTL_SEQUENCE); break;
    case SDLK_MINUS:	dvr_ctl(DVRCTL_MINUS);	break;
    case SDLK_EQUALS:   /* fallthrough */
    case SDLK_PLUS:	dvr_ctl(DVRCTL_PLUS);	break;
    case SDLK_PERIOD:	dvr_ctl(DVRCTL_SET);	break;
    case SDLK_z:	dvr_ctl(DVRCTL_ZOOM);	break;

    case SDLK_m:	dvr_ctl(DVRCTL_MENU);	break;
    case SDLK_UP:	dvr_ctl(DVRCTL_UP);	break;
    case SDLK_DOWN:	dvr_ctl(DVRCTL_DOWN);	break;
    case SDLK_LEFT:	dvr_ctl(DVRCTL_LEFT);	break;
    case SDLK_RIGHT:	dvr_ctl(DVRCTL_RIGHT);	break;
    case SDLK_RETURN:	dvr_ctl(DVRCTL_ENTER);	break;

    case SDLK_QUESTION: /* fallthrough */
    case SDLK_SLASH:	dvr_ctl(DVRCTL_SEARCH);	break;
    case SDLK_INSERT:	dvr_ctl(DVRCTL_RECORD);	break;
    case SDLK_p:	dvr_ctl(DVRCTL_PLAY);	break;
    case SDLK_s:	dvr_ctl(DVRCTL_STOP);	break;
    case SDLK_COMMA:	dvr_ctl(DVRCTL_SLOW);	break;
    case SDLK_SPACE:	dvr_ctl(DVRCTL_PAUSE);	break;
    case SDLK_PAGEDOWN:	dvr_ctl(DVRCTL_FAST);	break;
    case SDLK_PAGEUP:	dvr_ctl(DVRCTL_REW);	break;

    case SDLK_KP_MINUS:	dvr_ctl(DVRCTL_AUMINUS);	break;
    case SDLK_KP_PLUS:	dvr_ctl(DVRCTL_AUPLUS);	break;
    case SDLK_KP0:	dvr_ctl(DVRCTL_0);	break;
    case SDLK_KP1:	dvr_ctl(DVRCTL_1);	break;
    case SDLK_KP2:	dvr_ctl(DVRCTL_2);	break;
    case SDLK_KP3:	dvr_ctl(DVRCTL_3);	break;
    case SDLK_KP4:	dvr_ctl(DVRCTL_4);	break;
    case SDLK_KP5:	dvr_ctl(DVRCTL_5);	break;
    case SDLK_KP6:	dvr_ctl(DVRCTL_6);	break;
    case SDLK_KP7:	dvr_ctl(DVRCTL_7);	break;
    case SDLK_KP8:	dvr_ctl(DVRCTL_8);	break;
    case SDLK_KP9:	dvr_ctl(DVRCTL_9);	break;

    case SDLK_q:
      dvr_qual++;
      if (dvr_qual >= 3) {
        dvr_qual = 0;
      }
      dvr_setqual(dvr_qual);
      break;

    case SDLK_r:
      dvr_reso++;
      if (dvr_reso >= 3) {
        dvr_reso = 0;
      }
      dvr_setreso(dvr_reso);
      break;
    case SDLK_y:
      opt_showydouble = !opt_showydouble;
      break;
    default:
      /* nop */
     break;
  }
  return;
}

void
sdlevent(void)
{
  SDL_Event ev;

  while (SDL_PollEvent(&ev)) {
    switch (ev.type) {
      case SDL_QUIT:		/* User-requested quit */
        quit++;
        break;
      case SDL_KEYDOWN:		/* Keys pressed */
        sdlkeypress(&ev.key);
        break;
      case SDL_ACTIVEEVENT:	/* Application loses/gains visibility */
/*        if (ev.active.type == SDL_APPACTIVE) {
          notvisible = !ev.active.state;
        }
        break; fallthrouh XXX */
      case SDL_VIDEOEXPOSE:	/* Screen needs to be redrawn */
        SDL_Flip(screen);
        break;
      case SDL_KEYUP:		/* Keys released */
      case SDL_MOUSEMOTION:	/* Mouse moved */
      case SDL_MOUSEBUTTONDOWN:	/* Mouse button pressed */
      case SDL_MOUSEBUTTONUP:	/* Mouse button released */
      case SDL_JOYAXISMOTION:	/* Joystick axis motion */
      case SDL_JOYBALLMOTION:	/* Joystick trackball motion */
      case SDL_JOYHATMOTION:	/* Joystick hat position change */
      case SDL_JOYBUTTONDOWN:	/* Joystick button pressed */
      case SDL_JOYBUTTONUP:	/* Joystick button released */
      case SDL_SYSWMEVENT:	/* System specific event */
      case SDL_VIDEORESIZE:	/* User resized video mode */
      case SDL_USEREVENT:	/* User defined event */
      default:
        /* nop */
        break;
    } /* switch (event.type) */
  } /* while (SDL_PollEvent(&ev) */
}

void
sdlmodes(SDL_Rect **modes)
{
  int i;

  /* Check if there are any modes available */
  if (!modes) {
    puts("  No modes available");
    return;
  }
  /* Check if our resolution is restricted */
  if (modes == (SDL_Rect **)-1) {
    puts("  All resolutions available");
    return;
  } 
  /* Print valid modes */
  for(i=0; modes[i]; ++i) {
    printf("  %4d x %4d\n", modes[i]->w, modes[i]->h);
  }
}

void
sdlvideoinfo(void)
{
  const SDL_VideoInfo *vi;
  SDL_Rect **modes;
  char buf[256];

  if (SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) {
    fprintf(stderr, "%s: SDL_InitSubSystem(SDL_INIT_VIDEO): %s\n", 
        argv0, SDL_GetError());
    return;
  }
  printf("Video driver: %s\n", SDL_VideoDriverName(buf, sizeof buf));

  vi = SDL_GetVideoInfo();

#define YN(x) ((x) ? "yes" : "no")

  printf("Hardware acceleration: %s\n", YN(vi->hw_available));
  printf("Window manager: %s\n", YN(vi->wm_available));
  if (vi->hw_available) {
    printf("HW -> HW blits accelerated: %s\n", YN(vi->blit_hw));
    printf("HW -> HW colorkey blits accelerated: %s\n", YN(vi->blit_hw_CC));
    printf("HW -> HW alpha blits accelerated: %s\n", YN(vi->blit_hw_A));
  }
  printf("SW -> HW blits accelerated: %s\n", YN(vi->blit_sw));
  printf("SW -> HW colorkey blits accelerated: %s\n", YN(vi->blit_sw_CC));
  printf("SW -> HW alpha blits accelerated: %s\n", YN(vi->blit_sw_A));
  printf("Color fills accelerated: %s\n", YN(vi->blit_fill));
  printf("Video memory (kB): %d\n", vi->video_mem);
  printf("Bits per pixel: %d\n", vi->vfmt->BitsPerPixel);
  printf("Current width: %d\n", vi->current_w);
  printf("Current height: %d\n", vi->current_h);

  if (vi->hw_available) {
    modes = SDL_ListModes(NULL, SDL_HWSURFACE | SDL_ANYFORMAT);
    if (modes) { puts("Available windowed HW modes:"); sdlmodes(modes); }

    modes = SDL_ListModes(NULL, SDL_HWSURFACE | SDL_FULLSCREEN | SDL_ANYFORMAT);
    if (modes) { puts("Available full screen HW modes:"); sdlmodes(modes); }
  }

  modes = SDL_ListModes(NULL, SDL_SWSURFACE | SDL_ANYFORMAT);
  if (modes) { puts("Available windowed SW modes:"); sdlmodes(modes); }

  modes = SDL_ListModes(NULL, SDL_SWSURFACE | SDL_FULLSCREEN | SDL_ANYFORMAT);
  if (modes) { puts("Available full screen SW modes:"); sdlmodes(modes); }

  return;
}

SDL_Surface *
sdlscaleydouble(SDL_Surface *i)
{
  SDL_Surface *ni;
  int y;

  if (!(ni = SDL_CreateRGBSurface(SDL_SWSURFACE, i->w, i->h * 2, 
      i->format->BitsPerPixel, 
      i->format->Rmask, i->format->Gmask, i->format->Bmask, 
      i->format->Amask))) {
    fprintf(stderr, "%s: SDL_CreateRGBSurface: %s\n", argv0, SDL_GetError());
    return NULL;
  }

  for (y = 0; y < i->h; y++) {
    memcpy((Uint8 *) ni->pixels + (y * 2) * ni->pitch,
        (Uint8 *) i->pixels + y * i->pitch,
        i->w * i->format->BytesPerPixel);
    memcpy((Uint8 *) ni->pixels + (y * 2 + 1) * ni->pitch,
        (Uint8 *) i->pixels + y * i->pitch,
        i->w * i->format->BytesPerPixel);
  }

  return ni;
}

int
showsdlframe(SDL_Surface *i)
{
  SDL_Surface *id;
  Uint32 vidfl;
  Uint32 ticks;
#ifdef DO_SDLDISPLAYFORMAT /* XXX */
  SDL_Surface *tmpid;
#endif

  if (screen == NULL || prevw != i->w 
      || prevh != i->h * (opt_showydouble ? 2 : 1)) { 
    if (SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) {
      fprintf(stderr, "%s: SDL_InitSubSystem(SDL_INIT_VIDEO): %s\n", 
          argv0, SDL_GetError());
      return -1;
    }
/* XXX
	window_icon = SDL_LoadBMP(data_path());

	if (window_icon == NULL) {

		die("*** SDL ERROR *** unable to load bitmap (%s)",
			SDL_GetError()
		);
	}

	SDL_WM_SetIcon(window_icon, NULL);

	SDL_FreeSurface(window_icon);
*/

    /* initialize screen display */
    vidfl = 0;
    
    vidfl |= (opt_showhw ? SDL_HWSURFACE : SDL_SWSURFACE);

    if (opt_showasync)		vidfl |= SDL_ASYNCBLIT;
    if (opt_showdoublebuf)	vidfl |= SDL_DOUBLEBUF;
    if (opt_showfullscreen)	vidfl |= SDL_FULLSCREEN;
    if (opt_shownoframe)	vidfl |= SDL_NOFRAME;

    if (!(screen = SDL_SetVideoMode(i->w, i->h * (opt_showydouble ? 2 : 1), 
        i->format->BitsPerPixel, /* bpp or 0 ? bpp yields better with x11 */
        vidfl))) {
      fprintf(stderr, "%s: SDL_SetVideoMode(%d, %d, ..): %s\n", 
          argv0, i->w, i->h, SDL_GetError());
      return -1;
    }
    if (opt_showtitle) {
      SDL_WM_SetCaption(opt_showtitle, opt_showtitle);
    }
  }

  if (opt_verbose >= 2) {
    ticks = SDL_GetTicks();
  }

  id = i;

  if (opt_showydouble) {
    if (!(id = sdlscaleydouble(id))) {
      return -1;
    }
#ifdef DO_SDLDISPLAYFORMAT /* XXX */
    tmpid = id;
#endif
  }

#ifdef DO_SDLDISPLAYFORMAT /* XXX */
  if (!(id = SDL_DisplayFormat(id))) {
    if (opt_showydouble) {
      SDL_FreeSurface(tmpid);
    }
    fprintf(stderr, "%s: SDL_DisplayFormat: %s\n", argv0, SDL_GetError());
    return -1;
  }
  if (opt_showydouble) {
    SDL_FreeSurface(tmpid);
  }
#endif

  if (SDL_BlitSurface(id, NULL, screen, NULL) < 0) {
    fprintf(stderr, "%s: SDL_BlitSurface: %s\n", argv0, SDL_GetError());
    return -1;
  }
  /* SDL_UpdateRect(screen, 0, 0, 0, 0); */
  SDL_Flip(screen); /* this is better in case we do double buffering */

  if (opt_verbose >= 2) {
    sdlticks += SDL_GetTicks() - ticks;
    sdlframes++;

    if ((sdlframes % 100) == 0) {
      printf("Average ticks per frame: %f      \r", 
          (double) sdlticks / sdlframes);
      fflush(stdout);
      sdlticks = 0;
      sdlframes = 0;
    }
  }

#ifdef DO_SDLDISPLAYFORMAT /* XXX */
  SDL_FreeSurface(id);
#else 
  if (opt_showydouble) {
    SDL_FreeSurface(id);
  }
#endif

  prevw = i->w;
  prevh = i->h * (opt_showydouble ? 2 : 1);

  if (!opt_threads) {
    sdlevent();
  }

  return 0;
}

int
showjpegframe(void * data, size_t datalen)
{
  SDL_Surface *image;
  SDL_RWops *rwop;
  int ret;

  if (notvisible) {
    if (!opt_threads) {
      sdlevent();
    }
    return 0;
  }

  if (!(rwop = SDL_RWFromMem(data, datalen))) {
    fprintf(stderr, "%s: SDL_RWFromMem: %s\n", argv0, SDL_GetError());
    return -1;
  }

  if (!(image = IMG_LoadJPG_RW(rwop))) {
    fprintf(stderr, "%s: IMG_LoadJPG_RW: %s\n", argv0, IMG_GetError());
    SDL_RWclose(rwop);
    return -1;
  }
  SDL_RWclose(rwop);

  ret = showsdlframe(image);

  SDL_FreeSurface(image);

  return ret;
}

/* eof */
