/*
 * snavtclient.c
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 */

static const char *svnid = "$Id: snavtclient.c 56 2008-08-28 12:37:00Z epipe $";

#include "snavtclient.h"

/* the following are set by main() */
char * argv0 = PACKAGE;

char * dvr_host = "127.0.0.1";
char * dvr_port = "80";
char * dvr_user = "admin";
char * dvr_pass = "admin";
int dvr_qual = 1; /* -1 = undef, 0 = high, 1 = med, 2 = low(dvr-default) */
int dvr_reso = 1; /* -1 = undef, 0 = big, 1 = med(dvr-default), 2 = small */
int dvr_chan = -1; /* -1 = undef */

enum snmode snmode = mode_none;

IPaddress dvr_ip = { INADDR_NONE, 0 };

int	opt_showasync = 0;
int	opt_showdoublebuf = 0;
int	opt_showfullscreen = 0;
int	opt_showhw = 0;
int	opt_shownoframe = 0;
int	opt_showskip = 0;
char *	opt_showtitle = NULL;
int	opt_showydouble = 0;
int	opt_threads = 0;
int	opt_verbose = 0;

/* subroutines */

void
license(void)
{
  puts(LICENSE);
  puts("");
  puts(COPYING);
}

void
version(void)
{
  const SDL_version * v = SDL_Linked_Version();

  printf("\n" PACKAGE_STRING " - AV TECH DVR Client\n\
\n\
Copyright (C) 2008 Janne Snabb\n\
\n\
This program comes with ABSOLUTELY NO WARRANTY. This is free software,\n\
and you are welcome to redistribute it under certain conditions.\n\
Please see the accompanying LICENSE and COPYING files or type\n\
\"%s --license\" for full license terms.\n\n", argv0);

  printf("Using SDL library version %u.%u.%u\n\n", 
      v->major, v->minor, v->patch);

  puts("Please see software web site for more information:\n");
  puts("  http://snavtclient.epipe.com/\n");
}

void 
usage(void)
{
  printf("usage: %s [options]\n\
  -c CHAN --channel=N       select video channel N\n\
  -j FILE --dumpjpeg=FILE   dump video frame(s) to JPEG FILE(s) (@ = seq)\n\
          --dumpcount=N     define how many frames to dump (default 1)\n\
          --dumpinterval=N  dump frame every N seconds (default 1)\n\
  -H HOST --dvrhost=HOST    DVR hostname or IP (default 127.0.0.1)\n\
  -P PORT --dvrport=PORT    DVR port (default 80)\n\
  -u USER --dvruser=USER    DVR login username (default admin)\n\
  -p PASS --dvrpass=PASS    DVR login password (default admin)\n\
  -p :    --dvrpass=:       prompt user for DVR login password on /dev/tty\n\
  -h      --help            print this usage and exit\n\
  -i      --info            show various information about the DVR device\n\
  -L      --license         display the full software license\n\
  -q QUAL --quality=QUAL    set image quality (0 = hi, 1 = med, 2 = lo)\n\
  -r RESO --resolution=RESO set image resolution (0 = hi, 1 = med, 2 = lo)\n\
  -d      --setdate         set DVR date and time from system time\n\
  -s      --show            show video stream on screen\n\
          --showasync       use asynchronous surface updates\n\
          --showdoublebuf   use double buffering for surface updates\n\
  -F      --showfullscreen  use full screen mode\n\
          --showhw          use hardware surfaces\n\
          --shownoframe     do not display window decorations\n\
  -S SKIP --showskip=SKIP   skip SKIP frames after each displayed frame\n\
  -T TITL --showtitle=TITL  show TITL as window title\n\
  -Y      --showydouble     scale 2x height of the video frame\n\
  -v      --verbose         output details about actions to stdout\n\
  -V      --version         print version information and exit\n\
          --videoinfo       output details about video driver\n", argv0);
}
 
/* main */
int 
main(int argc, char **argv)
{
  int ch;

  static struct option longopts[] = {
    { "channel",	required_argument,	NULL,		'c' },
    { "dumpjpeg",	required_argument,	NULL,		'j' },
    { "dumpcount",	required_argument,	NULL,		256 },
    { "dumpinterval",	required_argument,	NULL,		257 },
    { "dvrhost",	required_argument,	NULL,		'H' },
    { "dvrport",	required_argument,	NULL,		'P' },
    { "dvruser",	required_argument,	NULL,		'u' },
    { "dvrpass",	required_argument,	NULL,		'p' },
    { "help",		no_argument,		NULL,		'h' },
    { "info",		no_argument,		NULL,		'i' },
    { "license",	no_argument,		NULL,		'L' },
    { "quality",	required_argument,	NULL,		'q' },
    { "resolution",	required_argument,	NULL,		'r' },
    { "setdate",	no_argument,		NULL,		'd' },
    { "show",		no_argument,		NULL,		's' },
    { "showasync",	no_argument,		NULL,		259 },
    { "showdoublebuf",	no_argument,		NULL,		260 },
    { "showfullscreen",	no_argument,		NULL,		'F' },
    { "showhw",		no_argument,		NULL,		261 },
    { "shownoframe",	no_argument,		NULL,		262 },
    { "showskip",	required_argument,	NULL,		'S' },
    { "showtitle",	required_argument,	NULL,		'T' },
    { "showydouble",	no_argument,		NULL,		'Y' },
    { "verbose",	no_argument,		NULL,		'v' },
    { "version",	no_argument,		NULL,		'V' },
    { "videoinfo",	no_argument,		NULL,		258 },
    { NULL,		0,			NULL,            0  }
  };

  /* set global pointer to argv[0] so that we can display it in subroutines */
  argv0 = argv[0];

  while ((ch = getopt_long(argc, argv, "c:j:H:P:u:p:hiLq:r:FdsS:T:YvV", 
      longopts, NULL)) != -1) {
    switch (ch) { 
      case 'c': /* --channel=CHAN */
        dvr_chan = atoi(optarg);
        break;
      case 'j': /* --dumpjpeg=FILE */
        snmode = mode_dumpjpeg;
        dumpjpegsuffix = optarg;
        /* '@' in file name is replaced with 6 digit sequence number */
        dumpjpeg = strsep(&dumpjpegsuffix, "@");
        if (dumpjpegsuffix) {
          dumpseq = 1;
        } else {
          dumpseq = 0;
        }
        break;
      case 256: /* --dumpcount=N */
        dumpcount = atoi(optarg);
        break;
      case 257: /* --dumpinterval=N */
        dumpinterval = atoi(optarg);
        break;
      case 'H': /* --dvrhost=HOST */
        dvr_host = optarg;
        break;
      case 'P': /* --dvrport=PORT */
        dvr_port = optarg;
        break;
      case 'u': /* --dvruser=USER */
        dvr_user = optarg;
        break;
      case 'p': /* --dvrpass=PASS */
        dvr_pass = optarg;
        break;
      case 'h': /* --help */
        usage();
        exit(EXIT_SUCCESS);
        break; /* notreached */
      case 'i': /* --info */
        snmode = mode_info;
        break;
      case 'L': /* --license */
        license();
        exit(EXIT_SUCCESS);
        break; /* notreached */
      case 'q': /* --quality=QUAL */
        dvr_qual = atoi(optarg);
        break;
      case 'r': /* --resolution=RESO */
        dvr_reso = atoi(optarg);
        break;
      case 'd': /* --setdate */
        snmode = mode_setdate;
        break;
      case 's': /* --show */
        snmode = mode_show;
        break;
      case 259: /* --showasync */
        opt_showasync++; 
        break;
      case 260: /* --showdoublebuf */
        opt_showdoublebuf++; 
        break;
      case 'F': /* --showfullscreen */
        opt_showfullscreen++; 
        break;
      case 261: /* --showhw */
        opt_showhw++; 
        break;
      case 262: /* --shownoframe */
        opt_shownoframe++; 
        break;
      case 'S': /* --showskip=N */
        opt_showskip = atoi(optarg);
        break;
      case 'T': /* --showtitle=TITL */
        opt_showtitle = optarg;
        break;
      case 'Y': /* --showydouble */
        opt_showydouble++; 
        break;
      case 'v': /* --verbose */
        opt_verbose++;
        break;
      case 'V': /* --version */
        version();
        exit(EXIT_SUCCESS);
        break; /* notreached */
      case 258: /* --videoinfo */
        sdlvideoinfo();
        exit(EXIT_SUCCESS);
        break; /* notreached */
      case 0: 
        usage();
        break; 
      default:
        usage();
        exit(EXIT_FAILURE);
    } /* switch (ch) */
  } /* while (getopt) */
  argc -= optind;
  argv += optind;

  if (argc) {
    usage();
    exit(EXIT_FAILURE);
  }

#if defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  if (strncmp(getpwuid(getuid())->pw_gecos, "Boris Salomon", 13) == 0) {
    puts("Die you liar!");
    exit(EXIT_FAILURE);
  }
#endif

  if (snmode == mode_none) {
    /* if we had a proper GUI for doing all the settings etc, we would 
       start it instead and ask the user where to connect... but we
       don't have that yet... */

    printf("Nothing to do. Run \"%s --help\" for usage information.\n", argv0);
    exit(EXIT_SUCCESS);
  }

  if (!strcmp(dvr_pass, ":")) {
    /* XXX Is getpass able to read more than 8 characters on other modern
       arhitectures than FreeBSD? Must check... */

    dvr_pass = getpass("Password: ");
  }
 
  if(SDL_Init(SDL_INIT_NOPARACHUTE) < 0) {
    fprintf(stderr, "%s: SDL_Init: %s\n", argv0, SDL_GetError());
    exit(EXIT_FAILURE);
  }
  atexit(&SDL_Quit);

  if(SDLNet_Init() < 0) {
    fprintf(stderr, "%s: SDLNet_Init: %s\n", argv0, SDLNet_GetError()); 
    exit(EXIT_FAILURE); 
  } 
  atexit(&SDLNet_Quit);
 
  if (SDLNet_ResolveHost(&dvr_ip, dvr_host, atoi(dvr_port)) < 0) {
    fprintf(stderr, "%s: SDLNet_ResolveHost: %s\n", argv0, SDLNet_GetError());
    exit(EXIT_FAILURE);
  }

  switch (snmode) {
    case mode_none:
      break;
    case mode_info:
      /* as a side effect dvr_login() prints out some stuff if mode == info */
      if (!dvr_login()) {
        fprintf(stderr, "%s: login to %s:%d failed\n", 
            argv0, dvr_host, atoi(dvr_port));
        exit(EXIT_FAILURE);
      }
      /* we already printed out most essential stuff in dvr_login()...
         here we should do GetSysCfg.cgi or Getdeviceinfo.cgi and get
         additional details... but we'll implement it latar, it is 
         more essential to get at least some output right now XXX */

      break;
    case mode_setdate:
      if (!dvr_login()) {
        fprintf(stderr, "%s: login to %s:%d failed\n", 
            argv0, dvr_host, atoi(dvr_port));
        exit(EXIT_FAILURE);
      }
      dvr_setdatenow();
      break;
    case mode_show:
    case mode_dumpjpeg:
      if (!dvr_login()) {
        fprintf(stderr, "%s: login to %s:%d failed\n", 
            argv0, dvr_host, atoi(dvr_port));
        exit(EXIT_FAILURE);
      }
      if (dvr_reso >= 0) {
        dvr_setreso(dvr_reso);
      }
      if (dvr_qual >= 0) {
        dvr_setqual(dvr_qual);
      }
      if (dvr_chan >= 0) {
        dvr_setchan(dvr_chan);
      }
      if (!dvr_getvideo_init()) {
        fprintf(stderr, "%s: initializing getting video failed\n", argv0);
        exit(EXIT_FAILURE);
      }
      if (snmode == mode_show && !opt_showtitle) { 
        if (asprintf(&opt_showtitle, "%s:%s - " PACKAGE_STRING, 
            dvr_host, dvr_port) < 0) {
          fprintf(stderr, "%s: asprintf: %s\n", argv0, strerror(errno));
          exit(EXIT_FAILURE);
        }
      }
      if (!dvr_getvideo()) {
        fprintf(stderr, "%s: getting video failed\n", argv0);
        exit(EXIT_FAILURE);
      }
      break;
    default:
      assert(("unknown mode", 0));
  }

  /* no need to call these as they have been registered with atexit() */
  /* SDLNet_Quit(); */
  /* SDL_Quit(); */
 
  return EXIT_SUCCESS;
}

/* eof */
