/*
 * snavtclient.h
 *
 * Copyright (C) 2008 Janne Snabb
 *
 * This file is part of snavtclient.
 *
 * snavtclient is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * snavtclient is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with snavtclient.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * In addition to the terms of the GNU Affero General Public License, the
 * following ADDITIONAL TERMS apply:
 *
 * a) You are required to preserve the original legal notices and 
 * author attributions in modified versions.
 *
 * b) You are prohibited to misrepresentate the origin of the covered 
 * work.
 *
 * c) You are required to mark modified versions in reasonable ways
 * as different from the original version.
 *
 * d) You are prohibited to use the names of licensors or authors
 * for publicity purposes.
 *
 *
 * This software may be available under different licensing terms.
 * Please contact the author for any inquiries.
 *
 * Author contact information:
 *   Janne Snabb
 *   snabb at epipe.com
 *
 * Software web site:
 *   http://snavtclient.epipe.com/
 *
 * $Id: snavtclient.h 56 2008-08-28 12:37:00Z epipe $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_ASSERT_H
#include <assert.h>
#endif

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_image.h"

/* global definitions */
#define RECVBUFFER 8192         /* default buffer size for network receive */
#define DEBUG 1

#ifdef WORDS_BIGENDIAN
#error not yet supported
#else
#define le2h16(x) (x)
#define le2h32(x) (x)
#define h2le16(x) (x)
#define h2le32(x) (x)
#endif

/* snavclient.c *************************************************************/

/* the following are set by main() */
extern char * argv0;
extern char * dvr_host;
extern char * dvr_port;
extern char * dvr_user;
extern char * dvr_pass;
extern int    dvr_qual;
extern int    dvr_reso;
extern IPaddress dvr_ip;

extern int    opt_showasync;
extern int    opt_showdoublebuf;
extern int    opt_showfullscreen;
extern int    opt_showhw;
extern int    opt_shownoframe;
extern int    opt_showskip;
extern char * opt_showtitle;
extern int    opt_showydouble;
extern int    opt_threads;
extern int    opt_verbose;

enum snmode { mode_none, mode_info, mode_setdate, mode_show, mode_dumpjpeg };
extern enum snmode snmode;

/* license.c ****************************************************************/

extern const char * LICENSE;
extern const char * COPYING;

/* http.c *******************************************************************/

int	mySDLNet_TCP_Recv(TCPsocket, void *, int);
size_t	send_http_request(TCPsocket, char *, void *, size_t);
size_t	recv_http_response(TCPsocket, char **, char **);

/* dvrctl.c *****************************************************************/

extern char * dvr_cookie;
extern char * dvr_firmware;
extern char * dvr_productid;
extern char * dvr_macaddress;
extern char * dvr_hostname;

int dvr_setreso(int);
int dvr_setqual(int);
int dvr_setchan(int);
int dvr_setdatenow(void);
int dvr_ctl(int);
char *	dvr_login(void);

#define DVRCTL_UNKNOWN	0x0000

#define _DVRCTL_BASE	0x0100
#define _DVRCTL_MK(x)	(_DVRCTL_BASE + (x))

#define DVRCTL_MENU	_DVRCTL_MK(1)
#define DVRCTL_UP	_DVRCTL_MK(2)
#define DVRCTL_DOWN	_DVRCTL_MK(3)
#define DVRCTL_LEFT	_DVRCTL_MK(4)
#define DVRCTL_RIGHT	_DVRCTL_MK(5)
#define DVRCTL_ENTER	_DVRCTL_MK(6)
#define DVRCTL_SEARCH	_DVRCTL_MK(7)
#define DVRCTL_ZOOM	_DVRCTL_MK(8)
#define DVRCTL_RECORD	_DVRCTL_MK(9)
#define DVRCTL_PLAY	_DVRCTL_MK(10)
#define DVRCTL_STOP	_DVRCTL_MK(11)
#define DVRCTL_SLOW	_DVRCTL_MK(12)
#define DVRCTL_PAUSE	_DVRCTL_MK(13)
#define DVRCTL_FAST	_DVRCTL_MK(14)
#define DVRCTL_REW	_DVRCTL_MK(15)
#define DVRCTL_4CUT	_DVRCTL_MK(16)
#define DVRCTL_9CUT	_DVRCTL_MK(18)
#define DVRCTL_16CUT	_DVRCTL_MK(19)
#define DVRCTL_MINUS	_DVRCTL_MK(20)
#define DVRCTL_PLUS	_DVRCTL_MK(21)
#define DVRCTL_SET	_DVRCTL_MK(22)
#define DVRCTL_SEQUENCE	_DVRCTL_MK(23)
#define DVRCTL_AUMINUS	_DVRCTL_MK(24)
#define DVRCTL_AUPLUS	_DVRCTL_MK(25)
#define DVRCTL_TIMER	_DVRCTL_MK(26)
#define DVRCTL_USBCON	_DVRCTL_MK(27)
#define DVRCTL_USBDIS	_DVRCTL_MK(28)
#define DVRCTL_PTZ	_DVRCTL_MK(29)
#define DVRCTL_PTZGOTO	_DVRCTL_MK(30)

#define DVRCTL_0	_DVRCTL_MK(100)
#define DVRCTL_1	_DVRCTL_MK(101)
#define DVRCTL_2	_DVRCTL_MK(102)
#define DVRCTL_3	_DVRCTL_MK(103)
#define DVRCTL_4	_DVRCTL_MK(104)
#define DVRCTL_5	_DVRCTL_MK(105)
#define DVRCTL_6	_DVRCTL_MK(106)
#define DVRCTL_7	_DVRCTL_MK(107)
#define DVRCTL_8	_DVRCTL_MK(108)
#define DVRCTL_9	_DVRCTL_MK(109)

#define _DVRCTL_CHMIN		0x4200
#define _DVRCTL_CHMAX		0x42ff
#define DVRCTL_CH(x)		(_DVRCTL_CHMIN + (x))
#define DVRCTL_IS_CH(x)		((x) >= _DVRCTL_CHMIN && (x) <= _DVRCTL_CHMAX)
#define DVRCTL_GET_CH(x)	(DVRCTL_IS_CH(x) ? ((x) - _DVRCTL_CHMIN) : 0)

/* dvrvideo.c ***************************************************************/

typedef struct {
  char *	name;
  uint32_t	bit;
  size_t	len;
  char		swap;
} dvrvideohdrdesc_t;

extern dvrvideohdrdesc_t dvrvideohdrdesc[];
extern char * dumpjpeg;
extern char * dumpjpegsuffix;
extern int    dumpcount;
extern int    dumpinterval;
extern int    dumpseq;
extern time_t dumplast;
extern int    quit;

char *	dumpjpegframe(void *, size_t);
int	dvr_getvideo_init(void);
int	dvr_getvideo(void);

/* sdlshow.c ****************************************************************/

void	sdlvideoinfo(void);
int	showjpegframe(void *, size_t);

/* eof */
